<?php
/**
 * Description of inoplugs_array
 *
 * @author Schoenmann Guenter
 * @version 1.0.0.0
 */
class inoplugs_array extends inoplugs_object
{

	public function __construct(inoplugs_version $my_version = null, inoplugs_version $requ_version = null)
	{
		parent::__construct($my_version, $requ_version);
	}
	/**
	 *	Checks each entry in the array and if this is an array:
	 *      1.) if more than 1 entry leave the array,
	 *  or  2.) if 1 entry and not an array change this array to the only element in the array
	 *	or	2.) if 1 entry which is an array itself leave this entry as an array and
	 *		    recursivly call this function and alter each element like described here
	 *
	 *	The output will result like follows:
	 *
	 *  <no array>          returns   parameter <no array>
	 *  array ( 1, 3, 5)  returns   array  (1, 3, 5)
	 *  array ()          returns   ''
	 *  array (  1 )      returns   1
	 *  array ( array (....) )   returns  array  (  array of result of recursive call with inner array )
	 * 
	 * @param array $array		array or entry in array to convert
	 * @return mixed			the altered entry like described above
	 */
	static public function array_convert_to_basetyp_recursiv(&$array)
	{
		if(!is_array($array))
			return $array;

		$cnt = count($array);

		if($cnt == 0)
			return '';	

		$new = array();
		foreach ($array as $key => &$value)
		{
			if(($cnt == 1) && (!is_array($value)))
			{
				$v = $value;
				unset ($value);
				return $v;
			}
			$new[$key] = inoplugs_array::array_convert_to_basetyp_recursiv($value);
		}
		unset ($value);
		return $new;
	}

	/**
	 *	Inserts the given array or object at position $insertat.
	 *	In case of nonnumeric keys, equal keys will be overridden by array_merge,
	 *	numeric keys are renumbered.
	 *
	 *	If $insert is not an array, it will get a numeric key by default.
	 *
	 *	If $arrsource is not an array, $arrsource is returned unchanged
	 * 
	 * @param array $arrsource		array, where the new value should be inserted
	 * @param mixed $arrinsert		array or object/value to insert
	 * @param int $insertat			zerobased index, where to insert (-1 at end)
	 * @param bool $keep_references	if true, the references to the objects are kept alive
	 * @return array
	 */
	static public function &array_insert_at(&$arrsource, &$arrinsert, &$insertat = -1, $keep_references = false)
	{
		if(!is_array($arrsource))
			return $arrsource;

		if (!is_bool($keep_references))
			$keep_references = false;

		if(!is_array($arrinsert))
		{
			$insert = array(&$arrinsert);
		}
		else
		{
			$insert = &$arrinsert;
		}

		if(!is_numeric($insertat))
			$insertat = -1;
		
		if($insertat == 0)
		{
			$newopt = array();
			$restopt = &$arrsource;
		}
		elseif(($insertat <= -1) || ($insertat >= count($arrsource)))
		{
			$newopt = &$arrsource;
			$restopt = array();
		}
		else
		{
			if(! $keep_references)
			{
				$newopt = array_slice($arrsource, 0, ($insertat));
				$restopt = array_slice($arrsource, $insertat);
			}
			else
			{
				$newopt = array();
				$restopt = array();
				$i = 0;
				foreach ($arrsource as $key => &$value)
				{
					($i <= $insertat) ? $newopt[$key] = &$value : $restopt[$key] = &$value;
					$i++;
				}
				unset ($value);
			}
		}
		if(! $keep_references)
		{
			$ret = array_merge($newopt, $insert, $restopt);
			return $ret;
		}

		$ret = array();
		$i = 0;
		for ($j = 0; $j < 3; $j++)
		{
			switch ($j)
			{
				case 0:
					$old = &$newopt;
					break;
				case 1:
					$old = &$insert;
					break;
				default:
					$old = &$restopt;
					break;
			}
			foreach ($old as $key => &$value)
			{
				if(is_numeric($key))
				{
					$ret[$i] = &$value;
					$i++;
				}
				else
				{
					$ret[$i] = &$value;
				}
			}
			unset ($value);
		}
		return $ret;
	}
}
?>
