<?php
/**
 *	Baseclass for Exceptions based on class Exception
 *
 * @author Schoenmann Guenter
 * @version 1.0.0.0
 */
class inoplugs_exception extends Exception
{
	const FATAL_ERROR = -1;
	const WARNING = -2;

	const val_FATAL_ERROR = 'FATAL ERROR: ';
	const val_WARNING = 'WARNING: ';
	const val_DEFAULT = 'Errorcode = ';
	const val_POSITION = 'Position: ';
	const val_UNKNOWN = ' *** No Exceptionmessage stored - refer to manual for Errorcode *** ';

	/**
	 * stores postitioninformation of the error
	 * 
	 * @var int|string
	 */
	protected $position;			

	/**
	 *	Constructor, passes parameter to baseclass Exception
	 *
	 * @param string $message				Messagestring
	 * @param int $code						Code of Error (FATAL_ERROR | WARNING)
	 * @param int|string $errorposition		Information, where the error occured
	 * @param Exception $previous			objet of previous exceptionobjet to build up list of subsequent errors
	 */
	public function __construct($message = "", $code = 0, $errorposition = 0, Exception $previous = null)
	{
		if(is_string($message) || is_numeric($message))
		{
			$message = htmlspecialchars($message);
		}
		else
		{
			$message = '';
		}

		if(is_string($code))
		{
			$code = htmlspecialchars($code);
		}
		parent::__construct($message, $code, $previous);
		$this->position = $errorposition;
	}

	/**
	 *	Destructor, baseclass has none !!
	 */
	public function __destruct()
	{
		unset($this->postition);
	}


	/**
	 *	Calls get_string() and concatenates all Exceptions by calling getPrevious().
	 *	The last object of the list (= first
	 *	exception) will be the first in the list, so the exceptions are returned
	 *	as they occured.
	 *
	 * @return array		Exceptionmessage 'code: Message'
	 */
	public function &get_messages ()
	{
		$arrret = array();
		$arrret[] = $this->get_string();
		$e = $this->getPrevious();
		while ($e != null)
		{
			($e instanceof inoplugs_exception) ? $str = $e->get_string() : $str = inoplugs_exception::get_string_exception($e);
			array_unshift($arrret, $str);
			$e = $e->getPrevious();
		}
		return $arrret;
	}


	/**
	 *	Composes a string with 'code: Message' from the current exception.
	 *
	 * @return string		the composed string
	 */
	public function &get_string()
	{		
		$ret = "";
		switch($this->getCode())
		{
			case inoplugs_exception::FATAL_ERROR:
				$ret .= inoplugs_exception::val_FATAL_ERROR;
				break;
			case inoplugs_exception::WARNING:
				$ret .= inoplugs_exception::val_WARNING;
				break;
			default:
				$ret .= inoplugs_exception::val_DEFAULT . $this->getCode() . ': ';
				break;
		}
		if(trim($this->getMessage()) == '')
		{
			$ret .= inoplugs_exception::val_UNKNOWN . ' ';
		}
		else
		{
			$ret .= $this->getMessage();
		}

		if(!empty ($this->position))
		{
			$ret .= ' - '.inoplugs_exception::val_POSITION.$this->position;
		}
		return $ret;
	}

	/**
	 *	Gets a basic Exception - object and returns a minimal string for this object
	 *	without stackinformation:
	 *
	 *		getCode(), getMessage(), getFile(), getLine(), get_class()
	 *
	 * @param Exception $exeption			basic Exception or any derived class
	 * @return string				basi string without stack information
	 */
	static public function &get_string_exception (Exception $exeption)
	{
		return htmlspecialchars(sprintf("*** %1s (Code: %2d) - '%3s' occured in File '%4s' in Line %5d (object:'%6s')",inoplugs_exception::val_FATAL_ERROR, $exeption->getCode(), $exeption->getMessage(), $exeption->getFile(), $exeption->getLine(), get_class($exeption)));
	}

	/**
	 *	Scans all exceptions in this object and returns, if only warnings or fatal errors
	 *	exist. Only code = inoplugs_exception::WARNING results in the return of this value,
	 *	all other values result in FATAL_ERROR.
	 *
	 * @return int|false			inoplugs_exception::WARNING | inoplugs_exception::FATAL_ERROR; false, when no exceptions exist
	 */
	public function get_error_status()
	{
		$stat = false;
		
		$e = &$this;
		while ($e != null)
		{
			switch($e->getCode())
			{
				case inoplugs_exception::WARNING:
					if($stat === false)
						$stat = inoplugs_exception::WARNING;
					break;
				case inoplugs_exception::FATAL_ERROR:
				default:
					$stat = inoplugs_exception::FATAL_ERROR;
					break;
			}
			$e = $e->getPrevious();
		}
		return $stat;
	}

}

?>
