<?php
/**
 *	This class allows to summarise exeptions. It builds up an array
 *	or concatinates exceptions rendered to the class. It also provides
 *	methods for reporting the messages to the user.
 *
 * @author Schoenmann Guenter
 */
class inoplugs_exceptions extends inoplugs_object
{
	const val_FATAL_ERROR = '*** FATAL ERROR(S) FOUND ***';
	const val_WARNING = '*** WARNING(S) FOUND ***';
	const val_NO_ERRORS = '*** NO ERRORS FOUND ***';

	const FILE_CSS = 'exceptions.css';

	const CSS_CONTAINER = 'inoplugs-exception-container';
	const CSS_HEAD_DIV = 'inoplugs-exception-head-div';
	const CSS_HEAD_P = 'inoplugs-exception-head-p';
	const CSS_DIV = 'inoplugs-exception-div';
	const CSS_P = 'inoplugs-exception-p';
	const CSS_PRE = 'inoplugs-exception-pre';
	const CSS_PRE_ERROR = 'inoplugs-pre-error';
	const CSS_PRE_NORMAL = 'inoplugs-pre-normal';
	const CSS_PRE_OK = 'inoplugs-pre-ok';
	const CSS_INFO_ERROR = 'inoplugs-info-error';
	const CSS_INFO_OK = 'inoplugs-info-ok';
	const CSS_INFO_NORMAL = 'inoplugs-info-normal';

	public $css_container;				//	classes for styling output - can be set directly or via function parameters
	public $css_headdiv;				//	if more classes, seperate them by ' '
	public $tag_headp;					//	default is p
	public $css_headp;
	public $css_messagediv;
	public $tag_messagep;				//	default is p
	public $css_messagep;
			
	protected $exceptionlist;

	public function __construct()
	{
		parent::__construct();
		
		$this->css_container = self::CSS_CONTAINER;
		$this->css_headdiv = self::CSS_HEAD_DIV;
		$this->tag_headp = 'p';
		$this->css_headp = self::CSS_HEAD_P;
		$this->css_messagediv = self::CSS_DIV;
		$this->tag_messagep = 'p';
		$this->css_messagep = self::CSS_P;
		$this->exceptionlist = array();			//	numeric key => exception
	}

	public function __destruct()
	{
		unset ($this->exceptionlist);
	}

	/**
	 *	Removes all entries from the list of exceptions.
	 */
	public function clear()
	{
		$this->exceptionlist = array();
	}

	/**
	 *	Returns true, when exceptions exist. Reset forces the arraycounter to reset to
	 *	first element.
	 * 
	 * @param bool $reset	true, when to reset the loop to the beginning
	 * @return bool			true, when exceptions still exist
	 */
	public function have_exceptions($reset = false)
	{
		if(empty($this->exceptionlist))
				return false;
		
		if($reset)
		{
			reset($this->exceptionlist);
		}

		return (count($this->exceptionlist) > 0);
	}

	/**
	 *	Returns the current exception object and moves the internal arraypointer to the next.
	 *
	 * @return &inoplugs_exception		the current exception
	 */
	public function &get_exception()
	{
		$obj = current($this->exceptionlist);
		if($obj !== false)
		{
			next($this->exceptionlist);
		}
		return $obj;
	}

	/**
	 *	Adds a new exception to the exceptionlist at the end. Baseclass of the object must be
	 *	Exception. If a string is surrendered, a new Exception (Severe Error) is created with the
	 *	string, otherwise with a default message trying to convert the object to
	 *	string.
	 * 
	 * @param Exception $exception		Exceptionobject to add
	 */
	public function add_new($exception) 
	{
		$this->exceptionlist[] = $this->get_exception_object($exception);
		return;
	}

	/**
	 *	Adds a list of exception(s) to the existing. If an ivalid object is given,
	 *	a fatal errorexception is added trying to convert the object to a string..
	 *
	 * @param inoplugs_exceptions|inoplugs_exception $exceptions		inoplugs_exception or inoplugs_exceptions object to add to exosting list.
	 */
	public function add_exceptions(&$exceptions = null)
	{
		if(is_null($exceptions))
			return;

		if($exceptions instanceof inoplugs_exception || (!($exceptions instanceof inoplugs_exceptions)))
		{
			$this->add_new($exceptions);
			return;
		}

		$exceptions->have_exceptions(true);
		while($exceptions->have_exceptions())
		{
			$this->exceptionlist[] = $exceptions->get_exception();
		}
		return;
	}

	/**
	 *	Returns the last exceptionobject or null (needed for lining up Exceptions)
	 * 
	 * @return &inoplugs_exception		the last exceptiomobject or null
	 */
	public function &get_last()
	{
		$i = count($this->exceptionlist);
		if($i == 0)
		{
			return null;
		}
		return $this->exceptionlist[$i - 1];
	}


	/**
	 *	Replaces the last entry with the new one (Mainly used for lining up Exceptions.
	 *	Make a previous call to get_last and create a new exception with the returned object
	 *	as the third parameter and call replace_last.
	 *
	 * @param Exception $exception			Exceptionobject to add
	 */
	public function replace_last(&$exception)
	{
		$i = count($this->exceptionlist);
		if($i > 0)
		{
			unset ($this->exceptionlist[$i - 1]);
		}
		$this->add_new($exception);
		return;
	}

	
	/**
	 *	Checks, wether $exception is a child of Exception. If not, a object
	 *	of inoplugs_exception is created and returnd.
	 *
	 * @param Exception $exception		Errorobject to test and convert
	 * @return &inoplugs_exception|&Exception		a new object, if $exception is not a child of Exception
	 */
	protected function &get_exception_object(&$exception)
	{
		if($exception instanceof Exception)
		{
			return $exception;
		}

		if(is_string($exception))
		{
			$e = new inoplugs_exception($exception, inoplugs_exception::FATAL_ERROR);
		}
		else
		{
			$str = print_r($exception, true);
			$e = new inoplugs_exception("unexprected object rendered to class inoplugs_exceptions: ".get_class($exception).". Dump = ".$str, inoplugs_exception::FATAL_ERROR);
		}
		return $e;
	}

	/**
	 *	Returns the content of the exceptionlist packed in HTML-Tags. For each Exception of type
	 *	inoplugs_exception (and children) in the list the function get_string() is called.
	 *
	 *	If no parameters 'class*' are given or empty, the coressponding classmembers are used.
	 *	Formatting is done by calling get_html_output();
	 *
	 * @param string $class_container		CSS-Class <div> for Errorcontainer
	 * @param string $class_headdiv			CSS-Class <div> for Headercontainer
	 * @param string $class_head			CSS-Class <p> for Headercontainer
	 * @param string $class_div				CSS-Class <div> for Single Message container
	 * @param string $class_p				CSS-Class <p> for Single Message
	 * @return string				the HTML-formatted output generated by get_html_output
	 */
	public function &get_html_exceptions($tag_head = '', $tag_mess = '', $class_container = '', $class_headdiv = '', $class_head = '', $class_div='', $class_p='')
	{
		$headline = $this->get_headlinetext();
		$arrmess = $this->get_strings();
		
		return $this->get_html_output($headline, $arrmess, $tag_head, $tag_mess, $class_container, $class_headdiv, $class_head, $class_div, $class_p);
	}

	/**
	 *	Returns the content of $headline and the stringarray $message packed in HTML. It is the standard output
	 *	where a <div>-container has a <div> for the headline and subsequent <div> for each entry of $message.
	 *
	 *	If no parameters 'class*' are given or empty, the default classmembers are used.
	 *
	 * @param string $headline				the headlinestring, if empty the <div> is skipped
	 * @param array|string $message			strings for the messages, if empty array skipped.
	 * @param string $class_container		CSS-Class <div> for Errorcontainer
	 * @param string $class_headdiv			CSS-Class <div> for Headercontainer
	 * @param string $tag_head				HTML-tag for Headline string
	 * @param string $class_head			CSS-Class <p> for Headercontainer
	 * @param string $class_messdiv			CSS-Class <div> for Single Message container
	 * @param string $tag_mess				HTML-tag for messagestring
	 * @param string $class_mess			CSS-Class <p> for Single Message
	 * @return string				the HTML-formatted output
	 */
	public function &get_html_output($headline, $message, $tag_head = '', $tag_mess = '', $class_container = '', $class_headdiv = '', $class_head = '', $class_messdiv='', $class_mess ='')
	{
		if(empty($class_container))
			 $class_container = $this->css_container;
		if(empty($class_headdiv))
			 $class_headdiv = $this->css_headdiv;
		if(empty($tag_head))
			 $tag_head = $this->tag_headp;
		if(empty($class_head))
			 $class_head = $this->css_headp.' '.$this->get_head_colour_class();
		if(empty($class_messdiv))
			 $class_messdiv = $this->css_messagediv;
		if(empty($tag_mess))
			 $tag_mess = $this->tag_messagep;
		if(empty($class_mess))
			 $class_mess = $this->css_messagep;

		$ret = inoplugs_html::html_addtag_class_open('div', $class_container).inoplugs_html::html_add_comment('  Begin of exceptionlist  ');

		$h = '';
		if(is_string($headline))
		{
			$h = trim($headline);
		}
		if(!empty($h))
		{
			$ret .= inoplugs_html::html_addtag_class_open('div', $class_headdiv);
			$ret .= inoplugs_html::html_addtag_class_open($tag_head, $class_head).$h.inoplugs_html::html_addtag_close($tag_head);
			$ret .= inoplugs_html::html_addtag_close('div');
		}

		$m = array();
		if(is_array($message))
		{
			$m = &$message;
		}
		elseif(is_string($message))
		{
			$m[] = $message;
		}
		
		foreach ($m as $s)
		{
			$ret .= inoplugs_html::html_addtag_class_open('div', $class_messdiv);
			$ret .= inoplugs_html::html_addtag_class_open($tag_mess, $class_mess).$s.inoplugs_html::html_addtag_close($tag_mess);
			$ret .= inoplugs_html::html_addtag_close('div');
		}

		$ret .= inoplugs_html::html_addtag_close('div').inoplugs_html::html_add_comment('  End of exceptionlist  ');
		return $ret;
	}

	/**
	 * Returns the default class for this Exceptionlist:
	 *		inoplugs_exceptions::CSS_INFO_ERROR for Fatal Error
	 *		inoplugs_exceptions::CSS_INFO_NORMAL for Warning
	 *  
	 *@return string		see description
	 */
	public function get_head_colour_class()
	{
		$col = inoplugs_exceptions::CSS_INFO_NORMAL;
		$stat = $this->get_error_status();
		if($stat == inoplugs_exception::FATAL_ERROR)
			$col = inoplugs_exceptions::CSS_INFO_ERROR;
		return $col;
	}

	/**
	 *	Scans all exceptions in this object and returns, if only warnings or fatal errors
	 *	exist. Only code = inoplugs_exception::WARNING results in the return of this value,
	 *	all other code values result in FATAL_ERROR.
	 *
	 *	Exeptionobjects not of type inoplugs_exception result in FATAL_ERROR.
	 *
	 * @return int|false			inoplugs_exception::WARNING | inoplugs_exception::FATAL_ERROR; false, when no exceptions exist
	 */
	public function get_error_status()
	{
		$stat = false;

		foreach ($this->exceptionlist as $value)
		{
			($value instanceof inoplugs_exception) ? $newstat = $value->get_error_status() : $newstat = inoplugs_exception::FATAL_ERROR;
			switch ($newstat)
			{
				case inoplugs_exception::WARNING:
					if($stat === false)
						$stat = inoplugs_exception::WARNING;
					break;
				case inoplugs_exception::FATAL_ERROR:
				default:
					$stat = inoplugs_exception::FATAL_ERROR;
					break;
			}
			if($stat == inoplugs_exception::FATAL_ERROR)
				break;
		}
		return $stat;
	}

	/**
	 *	Returns the Headlinetext for this Exceptionobject:
	 *		Fatal Errors  |  Warning   |   No Errormessages
	 *
	 * @return string
	 */
	public function get_headlinetext()
	{
		$stat = $this->get_error_status();

		if($stat === inoplugs_exception::FATAL_ERROR)
			$ret = inoplugs_exceptions::val_FATAL_ERROR;
		elseif($stat === inoplugs_exception::WARNING)
			$ret = inoplugs_exceptions::val_WARNING;
		else
			$ret = inoplugs_exceptions::val_NO_ERRORS;

		return $ret;
	}
	/**
	 *	Returns the array of strings returned by each stored Exception in the correct order as
	 *	added to the list.
	 *
	 * @return array		array of all composed strings of each entry
	 */
	public function &get_strings()
	{
		$arr = array();
		foreach ($this->exceptionlist as &$value)
		{
			if($value instanceof inoplugs_exception)
			{
				$arr = array_merge($arr, $value->get_messages());
			}
			else
			{
				while ($value != null)
				{
					$arr[] = inoplugs_exception::get_string_exception($value);
					$value = $value->getPrevious();
				}
			}
		}
		unset($value);
		return $arr;
	}
}
?>
