<?php
/* 
 * Baseclass for all objects defined by inoplugs
 * ===============================================
 *
 * Contains base functionality for all custom objects, especially versioning
 *
 * @author Schoenmann Guenter
 * @version 1.0.0.0
 */
class inoplugs_object {

	/**
	 * The current version of the uppermost object. If there are more objects
	 * derived, it is up to 
	 *
	 * @var inoplugs_version
	 */
	protected $my_version;

	/**
	 * The actual requested version
	 * 
	 * @var inoplugs_version
	 */
	protected $requ_version;			//	version of requested object  array (4)


	/**
	 * Constructs the baseclass and sets the defaultvalues to 1.0.0.0
	 * 
	 * @param inoplugs_version $my_version
	 * @param inoplugs_version $requ_version 
	 */
	public function __construct(inoplugs_version $my_version = null, inoplugs_version $requ_version = null)
	{
		if(is_null($my_version))
		{
			$my_version = new inoplugs_version(array(1,0,0,0));
		}
		$this->my_version = $my_version;

		if(is_null($requ_version))
		{
			$requ_version = new inoplugs_version(array(1,0,0,0));
		}
		$this->requ_version = $requ_version;
	}
	
	public function  __destruct()
	{
		unset ($this->my_version);
		unset ($this->requ_version);
	}

	/**
	 * Sets my_version of the object
	 *
	 * @param inoplugs_version $newversion
	 */
	public function set_version(inoplugs_version $newversion)
	{
		$this->my_version = $newversion;
	}
	
	/**
	 * Sets requ_version of the object
	 *
	 * @param inoplugs_version $newversion
	 */
	public function set_required_version(inoplugs_version $newversion)
	{
		$this->requ_version = $newversion;
	}

	/**
	 * Checks, if this object supports the requested version. The standard
	 * behaviour checks, if the requested version equals or is lower than
	 * the actual version.
	 * If ne $requversion is given, $this->requ_version is taken.
	 *
	 * @param inoplugs_version $requversion
	 * @return bool		true, if $requversion <= $this->my_version
	 */
	public function supports_version(inoplugs_version $requversion = null)
	{
		if(is_null($requversion))
			$requversion = $this->requ_version;

		return $this->my_version->supports_version($requversion);
	}
	

	
}
?>
