<?php
/**
 * Description of inoplugs_Query
 *
 * @author Schoenmann Guenter
 * Version 1.0.0.0
 */

class inoplugs_query extends inoplugs_object
{
	/*
	 *	Contains the generated WP_Query objekt
	 */
	protected $queryvars;			//	array of query parameters to directly pass to WP
	protected $queryposts;			//	the result of the query from DB - returnd by WP-Query


	public $errors;					//	inoplugs_exceptions - object
	
	/**
	 *
	 * @param <type> $queryvars
	 */
	function __construct(&$queryvars = null)
	{
		parent::__construct();
		$this->queryvars = $queryvars;
		$this->errors = new inoplugs_exceptions();
		$this->queryposts = null;
	}

	function  __destruct()
	{
		unset($this->queryvars);
		unset($this->errors);
		unset($this->queryposts);
	}


	/**
	 *	Uses the function get_taxonomies and returns an array of all terms assosiated with the
	 *	taxonomies. $arg_get_tax can be used to select, what taxonomies are queried
	 *	(refer to http://codex.wordpress.org/Function_Reference/get_taxonomies)
	 *
	 *	If an array from a previous call is rendered, the new terms are added. If there is a
	 *	duplicate key (Term-ID), the previous one is overridden by the new one.
	 *
	 *	This function returns all active custom taxonomies:
	 *			$arg_get_tax = array ('public'   => true,  '_builtin' => false)
	 *
	 *	This function returns all built in taxonomies:
	 *			$arg_get_tax = array ('public'   => true,  '_builtin' => true)
	 *
	 * @param array $arg_get_tax		parameter for taxonomies to query
	 * @param array $old_terms			an array from a previous call, added with the new terms
	 * @return &array			"term_id" => StdClass object (returned by get_terms)
	 */
	static public function &get_termlist_and_taxonomies($arg_get_tax, $old_terms = null)
	{
		$output = 'names'; // or objects
		$operator = 'and'; // 'and' or 'or'
		$taxes = get_taxonomies($arg_get_tax, $output, $operator);

		if(!is_array($old_terms))			//	in case a wrong structure was given - ore nothing
		{
			$terms = array();
		}
		else
		{
			$terms = $old_terms;
		}
		
		if (is_array($taxes))
		{
			foreach ($taxes as $tax)
			{
				$t = get_terms($tax);
				if(is_array($t) && (count($t) > 0))
				{
					foreach ($t as $value)
					{
						$terms[intval($value->term_id)] = $value;
					}
				}
			}
		}
		return $terms;
	}


	/**
	 *	Gets the taxonomyname to a given Term and puts the TermID it in the array $info.
	 *	$info has the following structure:
	 *			"taxonomyname" => array (TermID, ....)
	 *
	 *	This function is used to split an array of Terms into the corresponding taxonomies.
	 * 
	 * @param string|int $term		TermID, slug or name of a category to get the ID
	 * @param array $terms			array returned by get_termlist_and_taxonomies
	 * @param array $info			the terms ordered into the taxonomy, they belong to
	 * @param bool $isname			true, when $term is the name of the term, otherwise slug is taken
	 * @return bool					true, when term was added to $info,
	 *								false, when not (possible for custom taxonomy not supported by active theme !!!)
	 */
	static public function get_taxinfo_from_list($term, $terms, &$info, $isname = false)
	{
		if(empty ($term))
		{
			return true;
		}

		$t = null;
		if(is_numeric($term))
		{
			$t = $terms[$term];
		}
		else
		{
			$n = strtolower($term);
			foreach ($terms as &$t1)
			{
				(!$isname) ? $nt = strtolower($t1->slug): $nt = strtolower($t1->name);
				if($n == $nt)
				{
					$t = $t1;
					break;
				}
			}
			unset($t1);
		}
		if(is_null($t))
			return false;

		$s = &$info[$t->taxonomy];
		if(is_null($s))
		{
			$info[$t->taxonomy] = array(intval($t->term_id));
		}
		else
		{
			$s[] = intval($t->term_id);
		}
		return true;
	}

	/**
	 *	Gets all Terms to a given taxonomyname and puts the TermID in the array $info.
	 *	$info has the following structure:
	 *			"taxonomyname" => array (TermID, ....)
	 * 
	 * @param string|int $tax		Taxonomyname or ID to get all the ID's of Terms, that belong to this taxonomy
	 * @param array $terms			array returned by get_termlist_and_taxonomies
	 * @param array &$info			the terms ordered into the taxonomy, they belong to
	 * @return bool					true, when term was added to $info,
	 *								false, when not (possible for custom taxonomy not supported by active theme !!!)
	 */
	static public function get_all_terms_from_list($tax, $terms, &$info)
	{
		if(empty ($tax))
		{
			return true;
		}

		$t = null;
		$arrid = array();
		$isnum = is_numeric($tax);
		if(!$isnum)
			$tax = strtolower($tax);
		$taxname = "";
		
		foreach ($terms as &$t1)
		{
			if($isnum)
			{
				if($t1->term_taxonomy_id == $tax)
				{
					$arrid[] = intval($t1->term_id);
					$taxname = &$t1->taxonomy;
				}

			}
			else
			{
				if(strtolower($t1->taxonomy) == $tax)
				{
					$arrid[] = intval($t1->term_id);
					$taxname = &$t1->taxonomy;
				}
			}
		}
		unset($t1);
		
		if(count($arrid) == 0)
			return false;

		$s = &$info[$taxname];
		if(! is_null($s))
		{
			$arrid = array_merge($s, $arrid);
		}
		$info[$taxname] = $arrid;
		return true;
	}

	/**
	 * Returns an array with distinct taxonomienames. It depends on $args.
	 * if custom or built in are selected and returned.
	 * This function is basic for get_the_term_list to retrieve links for custom taxonomies.
	 *
	 * Calls get_termlist_and_taxonomies for basic sekection.
	 *
	 * @param array $arg_get_tax		parameter for taxonomies to query
	 * @return array			stringarray of unique taxonomienames
	 */
	static public function &get_distinct_taxnames($args)
	{
		$names = array();
		$terms = self::get_termlist_and_taxonomies($args);

		foreach ($terms as $key => &$term)
		{
			if(!in_array($term->taxonomy, $names))
			{
				$names[] = $term->taxonomy;
			}
		}
		return $names;
	}

	/**
	 *	Returns a valid "Operator" for a tax query call:
	 *		IN = default
	 *		NOT IN
	 * 
	 * @param string $operator
	 */
	static public function validate_tax_query_operator($operator = '')
	{
		$valid = array('LIKE', 'NOT LIKE', 'IN', 'NOT IN', 'BETWEEN', 'NOT BETWEEN');
//		$valid = array('IN', 'NOT IN');
		
		$default = "IN";
		$operator = strtoupper($operator);

		if(empty($operator))
			return $default;

		if(in_array($operator, $valid))
				return $operator;
		
		return $default;
	}


	/**
	 *	Returns a valid "relation" for a tax query call:
	 *		OR = default
	 *		AND
	 *
	 * @param string $operator
	 */
	static public function validate_tax_query_relation($relation = '')
	{
		$valid = array('AND', 'OR');

		$default = "OR";
		$relation = strtoupper($relation);

		if(empty($relation))
			return $default;

		if(in_array($relation, $valid))
				return $relation;
	
		return $default;
	}
	/**
	 *
	 * @param array $queryarray			parsed array ready to pass directly to WP
	 * @return int		number of posts recieved from DB
	 */
	public function query(&$queryarray = null)
	{
		if(!is_null($queryarray))
		{
			$this->queryvars = $queryarray;
		}
		
		if(empty($this->queryvars) || (!is_array($this->queryvars)))
		{
			$e = new inoplugs_exception('Empty Array or illegal objet passed to inoplugs_query->query', inoplugs_exception::FATAL_ERROR);
			$this->errors->add_new($e);
			return false;
		}

		add_filter( 'posts_where' , array(&$this, 'handler_posts_where'), 10, 2);
		add_filter( 'posts_join' , array(&$this, 'handler_posts_join'), 10, 2);

		$this->queryposts = new WP_Query($this->queryvars);
		
		remove_filter('posts_where' , array(&$this, 'handler_posts_where'), 10, 2);
		remove_filter('posts_join' , array(&$this, 'handler_posts_join'), 10, 2);

		return $this->queryposts->post_count;

	}


	/**
	 *	Whether there are more posts available in the loop.
	 *
	 * @return bool			true, if more posts are available
	 */
	public function have_posts()
	{
		if(empty ($this->queryposts))
			return false;

		return $this->queryposts->have_posts();
	}

	/**
	 *	Set up the next post and iterate current post index.
	 *	Sets up the values for the global valiable $post
	 * 
	 * @return object		the next available post, which gets the current, null if no more
	 */
	public function &next_post()
	{
		if(empty ($this->queryposts))
			return null;

		$this->queryposts->the_post();
		return $this->queryposts->post;
	}

	/**
	 *	Restores global variable to the main query post. Should always be called when
	 *	finished with this query. It is not called when the object is destructed.
	 */
	public function reset_main_query()
	{
		wp_reset_postdata();
	}

	/**
	 * Returns the most recent SQL-Statement
	 *
	 * @return string		SQL-Statement or '', if no object
	 */
	public function &get_sql_statement()
	{
		$ret = '';
		if(!empty ($this->queryposts) && isset($this->queryposts->request))
			$ret = $this->queryposts->request;
		return $ret;
	}

	/**
	 *	Handlerfunction for WHERE - Clause.
	 *
	 *	Is called before DB-access
	 * 
	 * @global object $post
	 * @global <type> $wpdb
	 * @param string $where			the caluse to be altered (add your string with AND....)
	 * @param WP_Query $wpquery		the current wp-query object with the parsed query parameters
	 * @return string			the altered resp. original WHERE clause
	 */
    public function handler_posts_where($where, $wpquery)
    {
        global $post, $wpdb;

        return $where;
    }


    /*
     *  INNER JOIN wp1_term_relationships ON (wp1_posts.ID = wp1_term_relationships.object_id)
     *  INNER JOIN wp1_term_taxonomy ON (wp1_term_relationships.term_taxonomy_id = wp1_term_taxonomy.term_taxonomy_id)
     */

	/*
	 *		Handlerfunction for JOIN
	 */

	/**
	 *	Handlerfunction for JOIN
	 *	Is called before DB-access.
	 *	All joins are realised by INNER JOIN.
	 *
	 * @global object $post
	 * @global <type> $wpdb
	 * @param string $join			the caluse to be altered (add your string with AND....)
	 * @param WP_Query $wpquery		the current wp-query object with the parsed query parameters
	 * @return string			the altered resp. original JOIN clause
	 */
    public function handler_posts_join ($join, $wpquery)
    {
        global $post, $wpdb;

        return $join;
    }

}
?>
