<?php
/**
 * Description of inoplugs_stringparser
 *
 * @author Schoenmann Guenter
 * @version 1.0.0.0
 */
class inoplugs_stringparser extends inoplugs_object
{
	protected	$seperators,
				$keepnewline;

	/**
	 *		Standardconstructor
	 *
	 * @param array $arr_seperators		all the valid separator characters
	 * @param bool $b_keepnewline		if true, CR LF is kept in strings, otherwise removed
	 */
	public function __construct($arr_seperators = null, $b_keepnewline = null)
	{
		parent::__construct();
		
		if(is_array($arr_seperators))
			$this->seperators = $arr_seperators;
		else
			$this->seperators = array(",", "&", ";");

		(is_bool($b_keepnewline)) ? $this->keepnewline = $b_keepnewline : $this->keepnewline = false;
	}


	public function __destruct()
	{
		unset ($this->seperators);
	}



	/**
	 *	Gets a string and splits it in an array. Splitting takes place at the token
	 *	given in $sep resp. in $this->seperators.
	 *
	 *	The return array has the following strukture:
	 *		array[]  =  array  (	'pos' => startvalue in $querystring for errormessages
	 *								'value' => token
	 *							)
	 *
	 * @param string $querystring		string to parse and split
	 * @param array $arrstr				array of token and seperators
	 * @param array $sep				array of possible seperators. if given, it will
	 *									change the stored values from the constructor.
	 */
	public function get_token_array(&$querystring, &$arrstr, &$sep = null)
	{
		if(!empty($sep) && is_array($sep) )
		{
			$this->seperators = $sep;
		}

		$start = 0;
		$length = strlen($querystring);
		$end = 0;
		$token = "";
		$seperatetoken = "";

					//	copy string in array and prepare for parsing query
		while ($start < $length)
		{
			$this->get_string($querystring, $start, $end, $token, $seperatetoken);
			if(strlen($token) > 0)
			{
				$arrstr[] = array('pos' => $start, 'value' => $token);			//	allows errormesssages for syntax !!
			}
			if(strlen($seperatetoken) > 0)
			{
				$arrstr[] = array('pos' => $start, 'value' => $seperatetoken);
			}
			$start = $end;
		}
	}

	/**
	 *	Returns a string - token and starts scanning as $start. It accepts string as described in
	 *	documentation.
	 *
	 * @param string $string		the string to parse
	 * @param int $start			begin of parsing
	 * @param int $end				the next position to continue parsing
	 * @param string $token			the string-token found after start
	 * @param string $seperatetoken	the seperator character, when found a seperator, "" otherwise
	 */
	public function get_string(&$string, $start, &$end, &$token, &$seperatetoken)
	{
		$bret = false;
		$token = "";
		$seperatetoken = "";

		$bquote = false;
		$bquotechar = "";
		$toknew = "";

		$last = strlen($string) - 1;
		$i = $start;
		while ($i <= $last)
		{
			$c = $string[$i];
			(($i + 1) < $last) ? $cnext = $string[$i + 1] : $cnext = "";

			if(($c == "'") || ($c == "\""))		//	quoted string may beginn or end
			{
				if($bquote)						//	inside string
				{
					if($c == $bquotechar)		//	string ends - leave loop
					{
						$bquotechar = "";
						$bquote = false;
						break;
					}
					else
					{
						$toknew .= $c;			//	add other quote to string
					}
				}
				else
				{
					if($toknew == "")			//	string starts with quotes
					{
						$bquotechar = $c;
						$bquote = true;
					}
					else
					{
						$toknew .= $c;			//	add quote to unquoted string
					}
				}
			}
			else if($bquote)					//	inside quoted string
			{
				if($c == "\\")					//	\" resp. \'
				{
					if($cnext == $bquotechar)
					{
						$toknew .= $cnext;			//	add only quote to string
					}
					else
					{
						$toknew .= $c.$cnext;		//	add \quote
					}
					$i++;
				}
				else
				{
					$toknew .= $c;					//	add to string
				}
			}
			else						//	outside quoted string
			{
				$ctok = "";
				if($this->is_tokenseperator($c, $cnext, $ctok, $i))		//	check for end of token
				{
					$seperatetoken = $ctok;	
					break;
				}
				if($toknew == "")		//skip leading unprintables
				{
					if($this->is_starttext($c))
					{
						$toknew .= $c;
					}
				}
				else
				{
					if($this->keepnewline)		
					{
						$toknew .= $c;
					}
					else					//	skip \r \n
					{
						if(!(($c == "\r") || ($c == "\n")))
						{
							$toknew .= $c;
						}
					}
				}
			}
			$i++;
		}
		
		$end = ++$i;
		$token = trim($toknew);
	}

	/**
	 *	Checks, if the character is a seperator. If \ is used, either \x is returned, if no
	 *	seperator resp. x only, if a seperator. The position for continuing is incremented,
	 *	when \ is found.
	 *
	 * @param string $char				first character to check
	 * @param string $charnext			the next character
	 * @param string $token				returns the token
	 * @param int $charpos				returns the next position for continuing to parse
	 * @param array $char_seperators	seperatorarray if different from that rendered to constructer
	 * @return boolean					true, if returned token is a seperator
	 */
	protected function is_tokenseperator(&$char, &$charnext, &$token, &$charpos, &$char_seperators = null)
	{
		$sep = null;
				
		if(isset($char_seperators) && is_array($char_seperators))
		{
				$sep = &$char_seperators;
		}
		else
		{
			$sep = &$this->seperators;
		}

		if(is_null($sep))		//	no seperators defined - should not be actually
		{
			$token = $char;
			return false;
		}

		($char != "\\") ? $c = $char : $c = $charnext;

		$issep = in_array($c, $sep);

		if(!$issep)			//	no seperator found - copy character(s) to token
		{
			$token = $char;
			if ($char == "\\")
			{
				$token = $char.$charnext;
				$charpos++;
			}
		}
		else				//	seperator found
		{
			if ($char == "\\")		//	skip \ and ignore seperator
			{
				$token = $char.$charnext;
				$charpos++;
				$issep = false;
			}
			else
			{
				$token = $char;
			}
		}

		return $issep;
	}


	/**
	 *	Checks, if the character is a start of text
	 *	Allowed are printable chars.
	 *
	 * @param string $char		character to check
	 * @return bool				true, when this character is a start of a string (= any printable character)
	 */
	protected function is_starttext(&$char)
	{
		if(ctype_alnum($char))
			return true;

		if(ctype_graph($char))
			return true;

		return false;
	}
}
?>
