<?php
/**
 *	inoplugs_Version capsules Versioning and provides functions for handling
 *	different versions within a class. It can also be used to handle versionnumbers
 *	in applications.
 *
 * @author Schoenmann Guenter
 * @version 1.0.0.0
 */
class inoplugs_version
{
	/**
	 * version as array of int (4)
	 * 
	 * @var array		
	 */
	protected $version;				//

	/**
	 * Standardconstructor, which initialises the object to a version.
	 * Defaultversion is 1.0.0.0
	 *
	 * @param array|string|inoplugs_version $version		the version to be stored
	 */
	public function __construct($version = null)
	{
		$this->version = $this->parse_version_number($version);
	}

	/**
	 *		standarddestructor
	 */
	public function  __destruct()
	{
		unset ($this->version);
	}
	/**
	 *	Parses the variable $new_version and returns an array (0,0,0,0).
	 *	If the version is not correct, (0,0,0,0) is returned.
	 *
	 * @param array|string|inoplugs_version		$new_version	the versionnumber to parse
	 * @return array			array with version (0 => (int), 1 => (int), 2 => (int), 3 => (int))
	 */
	public function &parse_version_number($new_version)
	{
		if($new_version instanceof inoplugs_version)
			$this->version = $new_version;

		if(empty($new_version))
		{
			return array(1,0,0,0);
		}
		
		$vret = array();

		if(is_string($new_version))
		{
			$v = preg_split("/[\s,;.]+/", $new_version, null, PREG_SPLIT_NO_EMPTY);
		}
		else
		{
			$v = $new_version;
		}
		if(is_array($v))
		{
			$i = 0;
			while (($i < count($v)) && ($i < 4))
			{
				if((isset($v[$i])) && (is_numeric($v[$i])))
				{
					$vret[$i] = intval($v[$i]);
				}
				else
				{
					$vret[$i] = 0;
				}
				$i++;
			}
			while ($i < 4)
			{
				$vret[$i] = 0;
				$i++;
			}
			return $vret;
		}
		return array(1,0,0,0);
	}

	/**
	 *	Checks, if the given version is equal to the stored version.
	 *
	 * @param inoplugs_version|array|string $new_version		the version to be checked
	 * @return boolean			true, if the versions are equal
	 */
	public function is_equal_version($new_version)
	{
		$vtest = $this->parse_version_number($new_version);
		$a1 = array_diff_assoc ($this->version, $vtest);
		return (count($a1) == 0);
	}

	/**
	 * Checks, if $check_version equals or is lower than
	 * the version of this object..
	 *
	 * @param inoplugs_version $check_version
	 * @return bool		true, if $check_version <= $this->version
	 */
	public function supports_version(inoplugs_version $check_version)
	{
		$arrcheck = $check_version->version;
		foreach ($this->version as $key => $value)
		{
			if($value < $arrcheck[$key])
				return false;
			if($value > $arrcheck[$key])
				return true;
		}
		return true;		//	equal version
	}
}
?>
