<?php
/**
 *	This class capsules all the Functionality of the plugin and is the main file
 *	for the shortcode and for function calls.
 *
 * @author Schoenmann Guenter
 * @version 1.0.0.0
 */
class inoplugs_multiselect extends inoplugs_object
{
	const PLUGIN_MULTISELECT_VERSION = '1.0.0.0';
	const MULTISELECT_DB_VERSION = '1.0.0.0';
	
	protected $version ;			//	current version of this class - object inoplugs_version
	public $errors;					//	inoplugs_exceptions - object
	public $errors_parser;			//	HTML-string with Errors from inoplugs_parser
	protected $scode_atts;			//	the passed arguments to this plugin
	protected $scode_content;		//	the content of the shortcode in enclosed part
	public $shortcode_parsed;		//	array with splitted shortcode returned by parse_shortcode_multiselect()
	public $multiselect_query;		//	parsed query prepared for WP from shortcode
	public $pgs_query;				//	inoplugs_query object
	
	protected $id_css;				//	ID of the css extension for this query (string or int)
	protected $id_query;			//	ID of the stored query in DB
	protected $showposts;			//	true, if posts should be shown when 'categoryselection' has no possible posts
	protected $skin;				//	holds info about the skin (can only be set in adminpanel - default is light skin
	protected $addcontainer;		//	true, if tags with class should be added, otherwise only the content
									//	is added (template must contain the class definition for css.
	protected $showerrors;			//	true, if errors should be shown

	protected $classcont;			//	container for all
	protected $classcontid;
	protected $classskin;			//	skinclass of theme
	protected $classskinid;

	protected $parseerror;			//	parser error classes
	protected $parseerrorid;
	protected $parsenoerror;
	protected $parsenoerrorid;

	protected $classpre;
	protected $classpreid;

	/**
	 *		Standardconstructor
	 */
	public function __construct()
	{
		parent::__construct();
		
		$this->version = new inoplugs_version(self::PLUGIN_MULTISELECT_VERSION);
		$this->errors = new inoplugs_exceptions();
		$this->errors_parser = '';
		$this->scode_atts = null;
		$this->scode_content = null;
		$this->shortcode_parsed = null;
		$this->multiselect_query = null;
		$this->pgs_query = null;
		$this->id_css = '';
		$this->id_query = '';
		$this->showposts = false;
		$this->addcontainer = true;
		$this->showerrors = false;
		$this->skin = inoplugs_multiselect_const::CSS_SKIN_LIGHT;

		$this->classcont = '';
		$this->classcontid = '';
		$this->classskin = '';
		$this->classskinid = '';

		$this->parseerror = '';
		$this->parseerrorid = '';
		$this->parsenoerror = '';
		$this->parsenoerrorid = '';

		$this->classpre = '';
		$this->classpreid = '';

	}
	/**
	 *		Standarddestructor
	 */
	public function  __destruct()
	{
		unset ($this->version);
		unset ($this->errors_parser);
		unset ($this->errors);
		unset ($this->scode_atts);
		unset ($this->scode_content);
		unset ($this->shortcode_parsed);
		unset ($this->multiselect_query);
		unset ($this->pgs_query);
		unset ($this->id_css);
		unset ($this->id_query);
		unset ($this->showposts);
		unset ($this->addcontainer);
		unset ($this->showerrors);
		unset ($this->skin);
	}

	
	/**
	 *	Register the stylesheet and scripts to the <head> section
	 *
	 *	Only possible, before shortcode is called => static class
	 */
	static public function handler_wp_print_styles()
	{
		global $inoplugs_const;
		
		$urlpath = WP_PLUGIN_URL.'/'.str_replace(basename( __FILE__),'',plugin_basename(__FILE__));

		wp_register_style('inoplugs_multiselect_css', $urlpath . 'css/' . inoplugs_multiselect_const::FILE_CSS_MULTISELECT);
		wp_register_style('inoplugs_exceptions_css', $urlpath.'inoplugs_framework/css/'.inoplugs_exceptions::FILE_CSS);

		$custom = false;
		$filepath = str_replace(basename( __FILE__),'',__FILE__);
		$customcss = '$filepath/css/'.inoplugs_multiselect_const::FILE_CSS_MULTISELECT_CUSTOM;
		if(file_exists($customcss))
		{
			wp_register_style('inoplugs_multiselect_custom_css', $urlpath . 'css/' . inoplugs_multiselect_const::FILE_CSS_MULTISELECT_CUSTOM);
			$custom = true;
		}

		wp_enqueue_style('inoplugs_multiselect_css');
		wp_enqueue_style('inoplugs_exceptions_css');
		
		if($custom)
			wp_enqueue_style('inoplugs_multiselect_custom_css');


		if(!(is_bool($inoplugs_cred) && ($inoplugs_cred == false)))
		{
			add_action('wp_footer', array('inoplugs_multiselect', 'handler_wp_footer'));
		}
	}

	/*
	 * 
	 */
	static public function handler_wp_footer()
	{
		echo '<a class="ipms-none" target="_blank" href="http://inoplugs.com">Multiselect Ultimate Query Plugin by InoPlugs Web Design Vienna | Webdesign Wien and Juwelier Sch&ouml;nmann</a>';
		echo '<a class="ipms-none" target="_blank" href="http://schoenmann.at">Multiselect Ultimate Query Plugin by InoPlugs Web Design Vienna | Webdesign Wien and Juwelier Sch&ouml;nmann</a>';
	}

	/**
	 *	Handlerfunction for shortcode. 
	 *
	 *	This handler is necessary, because the shortcode does not cerate an instance
	 *	of this class. This is done here and the parameters are passed to this object.
	 *
	 * @param array $atts			parameteres in [] as strings, empty string when no parameters supplied
	 * @param string $content		text inside [xxx] content [\xxx]
	 * @param string $code			which shorctcodehandler called this (if multiple refere to same handler)
	 * @return string			the Outputstring to replace the shortcode - area in page
	 */
	static public function handler_multisel_shortcode($atts, $content=null, $code="")
	{
		$obj = null;
		isset($this) ? 	$obj = $this : $obj = new self;

		return $obj->multisel_shortcode($atts, $content, $code);
	}

	/**
	 *	Sets the unique ID for the css classes. This ID is added to the classnames to
	 *	provide a different styling of the query.
	 *
	 * @param string|int $id		unique ID, if int will be converted to string
	 * @return bool			true, if correct id could be set
	 */
	public function set_unique_id_for_css(&$id = "")
	{
		if(is_string($id))
		{
			$this->id_css = $id;
			return true;
		}
		if(is_numeric($id))
		{
			$this->id_css = (string) $id;
			return true;
		}
		return false;
	}

	/**
	 *	Sets, if the requested data should be packed in HTML-Tags with css class set.
	 *	If you set it to false, only the text will be put in the outputstream.
	 *
	 *  A non-bool value will result in true.
	 * 
	 * @param bool $add		true, when HTML-tags and class should be added
	 */
	public function set_add_containertags($add = true)
	{
		if(is_bool($add))
		{
			$this->addcontainer = $add;
		}
		else
		{
			$this->addcontainer = true;
		}
	}

	/**
	 *	This function implements the shortcode in classcontext.
	 *
	 * The following values for $atts are accepted:
	 *
	 *	[inoplugs_multiselect key="value", .....]
	 *
	 *			queryid=[string|int]
	 *			cssid=[string|int]
	 *			showposts=[ true|false|1|0]
	 *			error=[true|false|1|0]
	 *
	 *	Refere to documentation for the structure $content.
	 *
	 * @param array $atts			parameteres in [] as strings, empty string when no parameters supplied
	 * @param string $content		text inside [xxx] content [\xxx]
	 * @param string $code			which shorctcodehandler called this (if multiple refere to same handler)
	 * @return string			the Outputstring to replace the shortcode - area in page
	 */
	protected function &multisel_shortcode(&$atts, &$content=null, &$code="")
	{
		$this->scode_atts = shortcode_atts(array(inoplugs_multiselect_const::key_scode_cssid => '',
			                                     inoplugs_multiselect_const::key_scode_queryid => '',
												 inoplugs_multiselect_const::key_scode_showposts => 'false',
			                                     inoplugs_multiselect_const::key_scode_error => 'false'),
										$atts);

		if(empty(inoplugs_multiselect_const::$sectionkeys))
		{
			$tmp = new inoplugs_multiselect_const();		//	initialise static variables
		}
							//	load parameters - trust, that they are string arguments
		$this->id_css = trim(strtolower($this->scode_atts[inoplugs_multiselect_const::key_scode_cssid]));
		$this->id_query = trim(strtolower($this->scode_atts[inoplugs_multiselect_const::key_scode_queryid]));
		$this->showposts = trim(strtolower($this->scode_atts[inoplugs_multiselect_const::key_scode_showposts]));
		$this->showerrors = trim(strtolower($this->scode_atts[inoplugs_multiselect_const::key_scode_error]));
		$this->scode_content = &$content;

		(($this->showposts == 'true') || ($this->showposts == '1')) ? $this->showposts = true : $this->showposts = false;
		(($this->showerrors == 'true') || ($this->showerrors == '1')) ? $this->showerrors = true : $this->showerrors = false;

		$this->init_css_members();
		$output = "";
		
				//	if specified, load query from DB, ignore content but other attributes override DB
		if($this->id_query != '')
		{
			$e = new inoplugs_exception("Shortcode with ID '$this->id_query' does not exist in Database. This feature is only available in the Premium version of this plugin. Visit us at http://www.inoplugs.com for the other versions.", inoplugs_exception::FATAL_ERROR);
			$this->errors->add_new($e);
		}
		elseif(is_null($this->scode_content) || (strlen($this->scode_content) == 0))
		{
			$this->scode_content = "";
			$e = new inoplugs_exception("Empty querystring in shortcode given - please enter a query and display to get an output or remove shortcode.", inoplugs_exception::WARNING);
			$this->errors->add_new($e);
		}
		
		if(strlen($this->scode_content) > 0)
		{
			$output = $this->get_output_with_scode_content($this->scode_content);
		}
		
		if($this->errors->have_exceptions(true))
		{
			$output .= $this->errors->get_html_exceptions();
		}
		return $output;
	}

	/**
	 * The object $row contains all data from the option panel and checks it
	 * for syntax and sets the corresponding WP array and gets the actual SQL Statement.
	 *
	 * @param inoplugs_multiselect_row $row
	 * @return bool		
	 */
	public function check_querycode(inoplugs_multiselect_row &$row)
	{
		$this->init_from_db($row);
		
					//	analyse query and get errors set
		$this->get_output_with_scode_content($this->scode_content, false);
	}

	/**
	 * Initialises this object with the data supplied by a stored query
	 * 
	 * @param inoplugs_multiselect_row $row
	 * @param bool $adddisplay				true, if display section to be added
	 */
	protected function init_from_db(inoplugs_multiselect_row &$row, $adddisplay = false)
	{
		$this->id_css = $row->cssid;
		$this->id_query = $row->id;
		$this->showposts = $row->showposts;
		$this->showerrors = $row->showerrors;
		($row->background_theme == inoplugs_multiselect_row::BG_THEME_WHITE) ? $this->skin = inoplugs_multiselect_const::CSS_SKIN_LIGHT : $this->skin = inoplugs_multiselect_const::CSS_SKIN_DARK;
		$this->scode_content = $row->querystring;
		$this->init_css_members();

					//	build string equal to shortcode
		$this->scode_content = inoplugs_multiselect_const::section_query."\n";
		$this->scode_content .= $row->querystring."\n";
		$this->scode_content .= inoplugs_multiselect_const::section_display."\n";
		if(!$adddisplay)
			return;

		$this->scode_content .= implode("\n", $row->display);
	}

	/**
	 *	Analyzes the shortcode part, splits it in query and display, performs the
	 *	query in the DB and creates the outputstream for the replacement of the
	 *	shortode part.
	 *
	 *	If any errors, they are added to the classmember $this->errors.
	 * 
	 * @param string $content		the content-part of the shortcode
	 * @param bool $getoutput		if false, the output is not generated (used for syntaxcheck only)
	 * @return string		the result output for the page to replace the shortode section
	 */
	public function get_output_with_scode_content(&$content, $getoutput = true)
	{
		if(!is_bool($getoutput))
			$getoutput = true;

		$output = "";
		$err = "";
		$arrparsed = array();
		$parser = new inoplugs_multiselect_parser();
		$parser->css_error = "$this->parseerror $this->parseerrorid";
		$parser->css_noerror = "$this->parsenoerror $this->parsenoerrorid";
		
		$ok = $parser->parse_shortcode_multiselect($content, $arrparsed, $err);
		$this->errors_parser = &$err;
		$this->shortcode_parsed = $arrparsed;
		$this->multiselect_query = $this->set_queryvars_multiselect($this->shortcode_parsed[inoplugs_multiselect_const::section_query]);

		if($this->showerrors && $getoutput)		//	display content + generated query
		{
			$output .= inoplugs_html::html_addtag_class_open('div', "$this->classskin $this->classskinid $this->classcont $this->classcontid");
			if((!$ok) || (!empty ($this->errors_parser)))
			{
				$output .= $this->errors->get_html_output('ERROR(S)/WARNING(S) found in querystring:', $this->errors_parser, 'p', 'pre', '','','','',"$this->classpre $this->classpreid");
			}
			else
			{
				$str = inoplugs_multiselect_const::section_query.":\n"."$parser->str_query\n".inoplugs_multiselect_const::section_display.":\n$parser->str_display";
				$s = inoplugs_html::html_addtag_class_open('span', inoplugs_exceptions::CSS_PRE_NORMAL);
				$s .= htmlentities($str, ENT_QUOTES);
				$s .= inoplugs_html::html_addtag_close('span');
				$output .= $this->errors->get_html_output('Your Querystring:', $s, 'p', 'pre', '','','','',"$this->classpre $this->classpreid");
			}
			$output .= inoplugs_html::html_add_comment(' end of query shortcode ');

			if(!empty($this->multiselect_query)) 			//	output of generated query
			{
				$str = print_r($this->multiselect_query, true);
				$s = inoplugs_html::html_addtag_class_open('span', inoplugs_exceptions::CSS_PRE_NORMAL);
				$s .= htmlentities($str, ENT_QUOTES);
				$s .= inoplugs_html::html_addtag_close('span');

				$output .= $this->errors->get_html_output('Generated Query rendered to WP:', $s,'p', 'pre', '','','','',"$this->classpre $this->classpreid");
			}
			else
			{
				$output .= $this->errors->get_html_output('No generated Query for WP', 'Error in parsing querystring has occured');
			}
			$output .= inoplugs_html::html_addtag_close('div').inoplugs_html::html_add_comment(' end of errorcontainer ');
		}
		if($ok)
		{
			$this->query_by_array();
			if($this->have_posts() && $getoutput)
			{
				while($this->have_posts())
				{
					$p = $this->next_post();
					$output .= $this->get_output_for_post($p);
				}
			}
			$this->reset_main_query();
		}
		return $output;
	}

	/**
	 *	Recieves the parsed querystring from the content part of the shortcode handler and
	 *	forms the querystring, that can be accepted by WP
	 *
	 * @param array $arrquery		the parses querystring from the shortcode "content"-part
	 * @return array		the query, that can be accepted by WP query
	 */
	public function set_queryvars_multiselect(&$arrquery)
	{
		$wpq = array();

		if(is_null($arrquery) || empty ($arrquery) || !is_array($arrquery))
				return $wpq;
		if(count($arrquery) == 0)
				return $wpq;

					//	copy all key - valuepairs to WP - query except "categoryselection"
					//	Following changes have to be made (WP 3.1)
					//		'order' must be string
					//		"orderby" must be string seperated by ' '
		$catsel = null;
		$tax = null;
		foreach ($arrquery as $key => &$value)
		{
			$k = strtolower($key);

			if($k == inoplugs_multiselect_const::qu_catsel)	//	split later
			{
				$catsel = $value;
				continue;
			}
			elseif ($k == inoplugs_multiselect_const::wpqu_tax)	//	take unsplitted
			{
				$tax = $value;
				$wpq[$k] = inoplugs_array::array_convert_to_basetyp_recursiv($value);
				continue;
			}
										//	WP needs special formatting of parameters for some keywords -> do this
			if(($k == 'orderby') && (is_array($value)))
			{
				$v = implode(' ', $value);
			}
			elseif(($k == 'order') && (is_array($value)))		//	WP-bug - not able different sorting
			{
				$v = implode(',', $value);
			}
			elseif(($k == 'cat') && (is_array($value)))
			{
				$v = implode(',', $value);
			}
			else						//	multiple entries - leave them as array (hopefully it works)
			{
				$v = inoplugs_array::array_convert_to_basetyp_recursiv($value);
			}
			$wpq[$k] = $v;
		}
		unset ($value);

		if(is_null($catsel))
			return $wpq;
		
					//	if taxonomy set by caller - ignore "categoryselection"
		if(!is_null($tax))
		{
			$e = new inoplugs_exception("'".inoplugs_multiselect_const::wpqu_tax."' has been set by user. The '".inoplugs_multiselect_const::qu_catsel."' will be ignored.", inoplugs_exception::WARNING);
			$this->errors->add_new($e);
			return $wpq;
		}

		if(!is_array($catsel))
		{
			$e = new inoplugs_exception("'".inoplugs_multiselect_const::qu_catsel."' will be ignored, because it is not an array.", inoplugs_exception::WARNING);
			$this->errors->add_new($e);
			return $wpq;
		}
		
		$anz = count($catsel);
		if($anz == 0)
		{
			return $wpq;
		}

					//	get relation - if not specified or incorrect -> default OR
		(isset ($catsel[inoplugs_multiselect_const::qu_relation][0])) ? $rela = $catsel[inoplugs_multiselect_const::qu_relation][0] : $rela = '';
		$rel = inoplugs_query::validate_tax_query_relation($rela);
				
		
					//	get all active custom id's for the theme
		$args = array('public' => true,
				      '_builtin' => false);
		$terms = inoplugs_query::get_termlist_and_taxonomies($args);

					//	get all built in id's
		$args = array('public' => true,
				      '_builtin' => true);
		$terms = inoplugs_query::get_termlist_and_taxonomies($args, $terms);


		//	Split categoryselection into taxonomies and get the ID_Term (if slug or name is specified).
		$qu_tax = array();		//	takes all the arrays for taxonomie-query
								//	("relation" => "",
								//	array(.....)  )

		foreach ($catsel as $numkey => &$selection)
		{
			if(strtolower($numkey) == inoplugs_multiselect_const::qu_relation)
				continue;
			
			if(!is_numeric($numkey))		//	ignore any nonnumeric key (besides relation nothing allowed
			{
				$e = new inoplugs_exception("'$numkey' is not allowed as a keyword in '".inoplugs_multiselect_const::qu_catsel."' - will be ignored.", inoplugs_exception::WARNING);
				$this->errors->add_new($e);
				continue;
			}
			if(is_null($selection) || (!(is_array($selection))) || (count($selection) == 0))
			{
				continue;
			}

			isset ($selection[inoplugs_multiselect_const::key_operator][0]) ? $opa = $selection[inoplugs_multiselect_const::key_operator][0] : $opa = '';
			$op = inoplugs_query::validate_tax_query_operator($opa);
			
			$arrtax = array();			//	("taxonomiename" => array ("int", TermID))
			foreach ($selection as $key => &$arrval)
			{
				$sk = strtolower($key);
				
				if($sk == inoplugs_multiselect_const::key_operator)
					continue;

				foreach ($arrval as &$val)
				{
					switch($sk)
					{
						case inoplugs_multiselect_const::key_cats:
							inoplugs_query::get_taxinfo_from_list($val, $terms, $arrtax, false);
							break;
						case inoplugs_multiselect_const::key_catsname:
							inoplugs_query::get_taxinfo_from_list($val, $terms, $arrtax, true);
							break;
						case inoplugs_multiselect_const::key_taxs:
							inoplugs_query::get_all_terms_from_list($val, $terms, $arrtax);
							break;
						default:
							$e = new inoplugs_exception("'$key' is not supported as a keyword in '".inoplugs_multiselect_const::qu_catsel."' - will be ignored.", inoplugs_exception::WARNING);
							$this->errors->add_new($e);
					}
				}
				unset($val);
			}
			unset ($arrval);
			
			if(count($arrtax) == 0)
				continue;

							//	create array for taxonomy query
			$cnt_a = 0;
			foreach ($arrtax as $taxname => &$arrid)
			{
				$aid = array();
				foreach ($arrid as $valid)	
				{
					if(!in_array($valid, $aid))
					{
						$aid[] = $valid;
					}
				}
				if(count($aid)> 0)
				{
					$cnt_a++;
					$qu_tax[] = array(inoplugs_multiselect_const::wpqu_taxonomy => $taxname,
							 inoplugs_multiselect_const::wpqu_field => 'id',
							 inoplugs_multiselect_const::wpqu_terms => $aid,
							 inoplugs_multiselect_const::wpqu_operator => $op);
				}
				
			}
			unset ($arrid);

						//	if relation = AND and multiple entry in a select -> not possible (OR - AND combination !!!)
			if($rel == 'AND')
			{
				if($cnt_a > 1)
				{
					$e = new inoplugs_exception('It is not possible to have more than 1 taxonomie in a select when using relation = AND. Put each taxonomy in a single select. Relation will be changed to "OR". ', inoplugs_exception::WARNING);
					$this->errors->add_new($e);
					$rel = inoplugs_query::validate_tax_query_relation();
				}
			}
		}
		unset($selection);

		$cnt = count($qu_tax);
		if(($cnt == 0) && (!$this->showposts)) 			//  no matching cats found -> create empty query otherwise all posts are returned
		{
			$taxname = 'undef';
			if(count($terms) > 0)
			{
				$taxname = reset($terms)->taxonomy;
			}
			$qu_tax[] = array(inoplugs_multiselect_const::wpqu_taxonomy => $taxname,
							 inoplugs_multiselect_const::wpqu_field => 'id',
							 inoplugs_multiselect_const::wpqu_terms => array(0),
							 inoplugs_multiselect_const::wpqu_operator => 'IN');
		}
		elseif($cnt > 1)		//	relation only if more than 1 entry
		{
			$qu_tax[inoplugs_multiselect_const::wpqu_relation] = $rel;
		}

		if(count($qu_tax) > 0)
			$wpq[inoplugs_multiselect_const::wpqu_tax] = inoplugs_array::array_convert_to_basetyp_recursiv($qu_tax);

		return $wpq;
	}

	/**
	 *	Performs an independent query on the database with a correctly set array for WP.
	 *	If no parameter is passed, than the member $multiselect_query is used.
	 *
	 *	This function is the first used by queries stored in Database via the option page.
	 *
	 *	After a call to this function the loop has access to the posts.
	 * 
	 * @param array	$arrquery		the correct initialised array of queryarguments that can be passed directly to WP
	 * @return int				number of posts read from the database
	 *							-1 if the request could not be solved (referre to get_errors for errormessages)
	 */
	public function query_by_array(&$arrquery = null)
	{
		if(!is_null($arrquery))
		{
			$this->multiselect_query = $arrquery;
		}
		
		if(empty($this->multiselect_query) || (!is_array($this->multiselect_query)))
		{
			$e = new inoplugs_exception('Invalid or empty argument passed to funtion <query_by_array>', inoplugs_exception::FATAL_ERROR);
			$this->errors->add_new($e);
			return -1;
		}

		$this->pgs_query = new inoplugs_query($this->multiselect_query);
		$i = $this->pgs_query->query();
		return $i;
	}


	/**
	 *	Whether there are more posts available in the loop.
	 *
	 * @return bool			true, if more posts are available
	 */
	public function have_posts()
	{
		if(empty($this->pgs_query))
				return false;
		
		return $this->pgs_query->have_posts();
	}

	/**
	 *	Set up the next post and iterate current post index.
	 *	Sets up the internal values for the post by calling setup_postdata()
	 *
	 * @return object		the next available post, which gets the current, null if no more
	 */
	public function &next_post()
	{
		if(empty ($this->pgs_query))
			return null;

		return $this->pgs_query->next_post();
	}

	/**
	 *	Restores global variable to the main query post. Should always be called when
	 *	finished with this query. It is not called when the object is destructed.
	 */
	public function reset_main_query()
	{
		if(empty ($this->pgs_query))
			return;
		
		$this->pgs_query->reset_main_query();
	}

	/**
	 *	Builds up the output for the given post. Set_post_data() must have been
	 *	already called.
	 *
	 * @param object $post			the current postobject to get the info from
	 * @return sring			the output string enclosed in HTML-Tags of the current post
	 */
	public function get_output_for_post(&$post)
	{
		$o = "";
		$arrdisp = &$this->shortcode_parsed[inoplugs_multiselect_const::section_display];

					//	open enclosing <div> - container for post
		if($this->addcontainer)
		{
			$divclasspost = $this->classcont.'-'.inoplugs_multiselect_const::CSS_POST.'-'.$post->ID;

			$divclasspostid = '';
			if($this->id_css != "")
			{
				$divclasspostid = $divclasspost.'-'.$this->id_css;
			}
			$o .= inoplugs_html::html_addtag_class_open("div", "$this->classskin $this->classskinid $this->classcont $this->classcontid $divclasspost $divclasspostid");
		}
		if(count($arrdisp) == 0)
		{
			//	no displaypart defined - ?????
		}
		else
		{
			foreach ($arrdisp as $value)
			{
				$v = strtolower($value);
				if(in_array($v, inoplugs_multiselect_const::$displaykeys))
				{
					$o .= $this->get_tagged_post_content_by_keyword($post, $v);
				}
				else		//	leave unchanged from user
				{
					$o .= $value;
				}
			}
		}
					//	close enclosing <div> - container for post
		if($this->addcontainer)
		{
			$o .= inoplugs_html::html_addtag_close('div').inoplugs_html::html_add_comment(inoplugs_multiselect_const::CSS_PRE.inoplugs_multiselect_const::CSS_POST_CONTAINER);
		}

		return $o;
	}

	/**
	 *	Builds up the output for the given keyword. Set_post_data() must have been
	 *	already called.
	 *
	 * @param object $post			the current postobject to get the info from
	 * @param string $keyword		the keyword in lowercase to return the info
	 * @return sring			the output string enclosed in HTML-Tags of the current post
	 */
	protected function get_tagged_post_content_by_keyword(&$post, &$keyword)
	{
		$tx = "";
		$o = "";
		$tag_p = 'p';
		switch ($keyword)
		{
			case inoplugs_multiselect_const::DISP_TITLE:
				$divclass = inoplugs_multiselect_const::CSS_PRE.inoplugs_multiselect_const::CSS_DIV.inoplugs_multiselect_const::CSS_TITLE;
				$pclass = inoplugs_multiselect_const::CSS_PRE.inoplugs_multiselect_const::CSS_TITLE;
				$tx = get_the_title();
				$tag_p = 'h1';
				break;
			case inoplugs_multiselect_const::DISP_EXCERPT:
				$divclass = inoplugs_multiselect_const::CSS_PRE.inoplugs_multiselect_const::CSS_DIV.inoplugs_multiselect_const::CSS_EXCERPT;
				$pclass = inoplugs_multiselect_const::CSS_PRE.inoplugs_multiselect_const::CSS_EXCERPT;
				$tx = get_the_excerpt();
				break;
			case inoplugs_multiselect_const::DISP_DATE:
				$divclass = inoplugs_multiselect_const::CSS_PRE.inoplugs_multiselect_const::CSS_DIV.inoplugs_multiselect_const::CSS_DATE;
				$pclass = inoplugs_multiselect_const::CSS_PRE.inoplugs_multiselect_const::CSS_DATE;
				$tx = '<strong>Post Date: </strong>';
				$tx .= get_the_date('l, n. F. Y');
				$tag_p = '';
				break;
			default:
				$divclass = inoplugs_multiselect_const::CSS_PRE.inoplugs_multiselect_const::CSS_DIV.inoplugs_exceptions::CSS_INFO_ERROR;
				$pclass = inoplugs_multiselect_const::CSS_PRE.inoplugs_exceptions::CSS_INFO_ERROR;
				$tx = "Keyword '$keyword' has not been implemented yet or is invalid !!";
				break;
		}

		$divclassid = '';
		$pclassid = '';
		$divclasspost = '';
		$divclasspostid = '';
		if($this->addcontainer)
		{
			$divclasspost = inoplugs_multiselect_const::CSS_PRE.inoplugs_multiselect_const::CSS_POST.'-'.$post->ID;
			if($this->id_css != "")
			{
				$divclassid = $divclass.'-'.$this->id_css;
				$pclassid = $pclass.'-'.$this->id_css;
				$divclasspostid = $divclasspost.'-'.$this->id_css;
			}
			$o .= inoplugs_html::html_addtag_class_open("div", "$divclass $divclassid $divclasspost $divclasspostid");
			if($tag_p != '')
				$o .= inoplugs_html::html_addtag_class_open($tag_p, "$pclass $pclassid");
		}

		if(!empty ($tx))
			$o .= $tx;
		
		if($this->addcontainer)
		{
			if($tag_p != '')
				$o .= inoplugs_html::html_addtag_close($tag_p);
			$o .= inoplugs_html::html_addtag_close('div').inoplugs_html::html_add_comment("$divclass $pclassid");
		}

		return $o;
	}


	/**
	 *	Initializes all CSS-Class membervariables to have them ready for output. It also
	 *	initialises the Exceptionobjet for this class.
	 *
	 */
	protected function init_css_members()
	{
		$this->classcont = inoplugs_multiselect_const::CSS_PRE.inoplugs_multiselect_const::CSS_POST_CONTAINER;
		$this->classskin = inoplugs_multiselect_const::CSS_PRE.$this->skin;

		$this->classpre = inoplugs_exceptions::CSS_PRE;
		$this->parseerror = inoplugs_exceptions::CSS_PRE_ERROR;
		$this->parsenoerror = inoplugs_exceptions::CSS_PRE_NORMAL;

		if($this->id_css != '')
		{
			$this->classcontid = $this->classcont.'-'.$this->id_css;
			$this->classskinid = $this->classskin.'-'.$this->id_css;

			$this->classpreid = $this->classpre.'-'.$this->id_css;
			$this->parsenoerrorid = $this->parsenoerror.'-'.$this->id_css;
			$this->parseerrorid = $this->parseerror.'-'.$this->id_css;

							//	init exception object
			$this->errors->css_container .= ' '.$this->errors->css_container .'-'.$this->id_css;
			$this->errors->css_headdiv .= ' '.$this->errors->css_headdiv.'-'.$this->id_css;
			$this->errors->css_headp .= ' '.$this->errors->css_headp.'-'.$this->id_css;
			$this->errors->css_messagediv .= ' '.$this->errors->css_messagediv.'-'.$this->id_css;
			$this->errors->css_messagep .= ' '.$this->errors->css_messagep.'-'.$this->id_css;
		}
	}

}

?>
