<?php
/**
 *	This class provides functions for parsing strings from Userinput.
 *
 * @author Schoenmann Guenter
 * @version 1.0.0.0
 */
class inoplugs_multiselect_parser extends inoplugs_object
{
	const	WARNING_ERROR = 1;
	const	FATAL_ERROR = 2;

	public $str_query;				//	the querystring from Input with removed HTML and encoded special characters
	public $str_display;
	
	public $css_noerror;				//	classes for <span> for shwing errors
	public $css_error;
	
	protected $scqs_sep;			//	shortcode querystring - all possible seperators
	protected $scqs_sepstring;		//	shortcode querystring - possible seperators for unquoted strings

	/**
	 *	Constructor
	 */
	public function __construct()
	{
		parent::__construct();
		
		$this->scqs_sep = array();
		$this->scqs_sepstring = array();

		$this->str_query = '';
		$this->str_display = '';
		$this->css_noerror = '';
		$this->css_error = '';
	}

	/**
	 * Destructor
	 */
	public function __destruct() 
	{
	 unset($this->scqs_sep);
	 unset($this->scqs_sepstring);
	 
	 unset($this->str_query);
	 unset($this->str_display);
	 unset($this->css_noerror);
	 unset($this->css_error);
	}

	/**
	 *	Parses the content of the "content" - string in the encolsed shortcode.
	 *	The array &$result is filled with the data of the parse string and has the
	 *	following structure:
	 *
	 * array(
	 *		['query'] =>	array ( the splitted key-value pairs. This only represents the parsed values as described
	 *							in the documentation, not the WP-Query. It must be adjusted. )
	 *		['display'] =>  array ( numeric key - string splitted into text and keywords.
	 *							Each keyword has a single entry,
	 *							starts with $$ and can later be directly replaced by the content
	 *							from the database
	 *		)
	 *
	 *	See documentatiom for syntax.
	 *
	 *	@param string $content		the enclosed content to parse, is not altered
	 *	@param array $query			filled as dwescribed above
	 *	@param string $error		HTML - string appends and returns all errormessages that occured
	 *  @param string $css_noerror	CSS-Classes to add to <span> tag. If empty, public member $css_noerror is used
	 *  @param string $css_error	CSS-Classes to add to <span> tag. If empty, public member $css_error is used
	 *	@return	bool				true, if the string could be resolved to a valid query
	 *									  (a WARNING could still be in $error
	 *								false, if a fatal error occured
	 */
	public function parse_shortcode_multiselect(&$content, &$query, &$error, $css_noerror = '', $css_error = '')
	{
		$result = true;

		$arrsections = $this->split_sections($content);

		$query = array();

		if(isset($arrsections[inoplugs_multiselect_const::section_query]))
		{
			$str = html_entity_decode($arrsections[inoplugs_multiselect_const::section_query], ENT_QUOTES);
			$str = inoplugs_html::html_quots_decode($str);
			$str = strip_tags($str);
			$this->str_query = $str;

			$strqu = array();
			$result = $this->parse_shortcode_query_section($str, $strqu, $error, $css_noerror, $css_error);
			$query[inoplugs_multiselect_const::section_query] = $strqu;
		}

		if(isset($arrsections[inoplugs_multiselect_const::section_display]))
		{
			$str = &$arrsections[inoplugs_multiselect_const::section_display];
								//	WP may insert <p> tags in HTML-Editor - remove them
			$tags = array('p');
			$str = inoplugs_html::html_remove_tags_unpaired($str, $tags);
			$this->str_display = $str;
			
			$strdisp = array();
			$arrsplit = inoplugs_string::split_string_by_key($str, inoplugs_multiselect_const::$displaykeys, false);
			$query[inoplugs_multiselect_const::section_display] = $arrsplit;
		}
		return $result;
	}
	/**
	 *	Splits the string $content into the corresponding substrings defined in
	 *	inoplugs_multiselect_const::sectionkeys and places the substrings in the array.
	 *	No changes take place in the strings.
	 *
	 *	If more than 1 entry is found, they are concatenates into 1 string
	 *	( & for $$query$$, "" for $$display). This should actually not happen !!!
	 * 
	 * @param string $content		the enclosed content to parse, is not altered
	 * @return array				the splitted content, keys are same as in inoplugs_multiselect_const::sectionkeys
	 *								empty array when no matches found
	 */
	protected function split_sections(&$content)
	{
		$arrpos = array();
		$i = 0;

					//	find the possible positions to split
		foreach (inoplugs_multiselect_const::$sectionkeys as $key => $value)
		{
			$anf = 0;
			do
			{
				$i = stripos($content, $value, $anf);
				if($i !== false)
				{
					$arrpos[$i] = $value;
					$anf = ++$i;
				}

			} while ($i !== false);
		}

				//	no matches found - seems to be an error - returm empty array
		if(count($arrpos) == 0)
		{
			return array();
		}

		ksort($arrpos, SORT_NUMERIC);
				
		$arrsplited = array();

				//	split string
		reset($arrpos);

		while (current($arrpos) !== false)
		{
			$anf = key($arrpos);
			$key = current($arrpos);

			$keyn = next($arrpos);
			if($keyn === false)		//	end of array
			{
				$end = strlen($content);
			}
			else
			{
				$end = key($arrpos);
			}
			$anf += strlen($key);		//	skip keyword
			$len = $end - $anf;			//	length up to next keyword
			$st = substr($content, $anf, $len);
			if(isset($arrsplited[$key]))
			{
				if($key == inoplugs_multiselect_const::section_query)
					$arrsplited[$key] .= " & $st";
				else
					$arrsplited[$key] .= " $st";
			}
			else
			{
				$arrsplited[$key] = $st;
			}
		}
		return $arrsplited;
	}

	/**
	 *	Parses the string and splits the string in an array, which can be used later
	 *	to prepare the input for the query. Each string is put in an element of the array.
	 *
	 * @param string $querystring		String to parse ans split into array for
	 *									rendering to WP-Query object
	 * @param array $query			contains the complete splitted string, replaces any existing array
	 * @param string $error			Errormessage as string - currently no full support
	 * @param string $css_noerror	CSS-Classes to add to <span> tag. If empty, public member $css_noerror is used
	 * @param string $css_error		CSS-Classes to add to <span> tag. If empty, public member $css_error is used
	 * @return boolean				false, if a severe error ocured
	 *								true, if successfull or a warning only
	 */
	protected function parse_shortcode_query_section(&$querystring, &$query, &$error, &$css_noerror = '', &$css_error = '')
	{
		$this->scqs_sep = array('&','{','}','=',',',';');
		$this->scqs_sepstring = array(',',';');

		if(empty($css_noerror))
			$css_noerror = $this->css_noerror;
		if(empty($css_error))
			$css_error = $this->css_error;

		$sp = new inoplugs_stringparser($this->scqs_sep, false);

		$arrstr = array();
		$sp->get_token_array($querystring, $arrstr);

					//	strings are in array - parse and prepare query
		$query = array();		
		$start = 0;
		$isok = true;
		$err = array();
		$arrerr = array();
		$last = count($arrstr);
		do
		{			
			$isok = $this->get_line($arrstr, $start, $end, $query, $err);
			if(count($err) > 0)
			{
				$arrerr = array_merge($arrerr, $err);
				$err = array();
			}
			if((!$isok) || ($end >= $last))
				break;
			$start = $end;
		}
		while(true);

					//	 prepare errormessage
		if(count($arrerr) > 0)
		{
			$start = 0;
			$str = "";

			foreach ($arrerr as &$value)
			{
				$end = $arrstr[$value[0]]['pos'];
				$anz = $end - $start;

				$str.= inoplugs_html::html_addtag_class_open('span', $css_noerror);
				$str.= substr($querystring, $start, $anz);
				$str.=inoplugs_html::html_addtag_close('span');

				$str.= inoplugs_html::html_addtag_class_open('span', $css_error);

				if($value[1] == inoplugs_multiselect_parser::WARNING_ERROR)
				{
					$s1 = inoplugs_multiselect_const::err_warning;
				}
				else
				{
					$s1 = inoplugs_multiselect_const::err_error;
					$isok = false;
				}
				$str .= "\n$s1 ".$value[2];

				$str.=inoplugs_html::html_addtag_close('span');
				$start = $end;
			}
			if($end < strlen($querystring))
			{
				$str.= inoplugs_html::html_addtag_class_open('span', $css_noerror);
				$str.= substr($querystring, $end);
				$str.=inoplugs_html::html_addtag_close('span');
			}
			unset($value);
			$error .= $str;
		}
		return $isok;
	}


	/**
	 *	Parses the inputarray and adds an entry into $line:
	 *		- standardline = "Key" => "array of values"
	 *		- categoryselection =
	 *			array  ("taxs" => array(
	 *						"relation" => "array of values"
	 *						array
	 *							("Key" => "array of values"
	 *							 ....)
	 *						....
	 *				    )
	 *	You must check $end to know, wether the last line is reached !!
	 *
	 * @param array $strings	parsed stringarray <array> from content
	 * @param int $start		index, where to start
	 * @param int $end			index of next line
	 * @param array $line		the elements of the line are added as
	 *								"Key" => "array of values"
	 * @param array $error		errormessage (index, string) of found error
	 * @return bool			true, if line could be correctly passed or a warning only
	 *						false, if a severe error ocured
	 */
	protected function get_line(&$strings, $start, &$end, &$line, &$error)
	{
		$last = count($strings);
		if($start >= $last)			//	accept & as last character - place only a warning
		{
			$error[] = array($start, self::WARNING_ERROR, " & at end of string is ignored ! ");
			return true;
		}

		$eq = array();
		$amp = array();
		$sbraceo = array();			//	{ open
		$sbracec = array();			//	} close
		$dbracec = array();			//	{{ open
		$dbraceo = array();			//	}} open
		$sep = array();				//	array ("pos" => index of pos. in $strings, "sep" => "seperator")
		
		$sbopen = 0;
		$dbopen = 0;

		$contloop = true;		
		$i = $start;
		

		while ($i < $last)
		{
			($i + 1) < $last ? $next = $strings[$i + 1]['value'] : $next = "";
			$str = $strings[$i]['value'];
			switch ($str)
			{
				case "=":
					$eq[] = $i;
					$sep[] = array("pos" => $i, "sep" => $str);
					break;
				case "&":
					$amp[] = $i;
					$sep[] = array("pos" => $i, "sep" => $str);
					if($dbopen == 0)
					{
						$contloop = false;
					}
					break;
				case "{":
					$sb = $str;
					if(($next == "{") && ($dbopen == 0)) 		// "{" "{"  ->  "{{" exactly 1st occurene
					{
						$dbraceo[] = $i;
						$sb .= $next;
						$dbopen++;
						$i++;
					}
					else					//	"{" only
					{
						$sbraceo[] = $i;
						$sbopen++;
					}
					$sep[] = array("pos" => $i, "sep" => $sb);
					break;
				case "}":
					$sb = $str;
					if(($next == "}") && ($sbopen == 0))
					{
						$dbracec[] = $i;
						$sb .= $next;
						$dbopen--;
						$i++;
						$contloop = false;
						if(($i + 1) < $last)
						{
							if($strings[$i + 1]['value'] == "&")		//	accept & after }}
							{
								$i++;
							}
						}
					}
					else
					{
						$sbracec[] = $i;
						$sbopen--;
					}
					$sep[] = array("pos" => $i, "sep" => $sb);
					break;
				default:
				
					
			}						//	end of switch - statement

			if(!$contloop)
			{
				$end = ++$i;
				break;
			}
			$i++;
		}				//	end while  (scanning array)

		if(($i >= $last))		//	end of string reached - left loop
		{
			$end = $last;
		}

		if(count($eq) == 0)
		{
			$error[] = array($start, self::FATAL_ERROR, "Missing '=', rest of your text will be ignored! ");
			return false;
		}
		if($eq[0] == $start)
		{
			$error[] = array($start, self::FATAL_ERROR, "Missing key before '=', rest of your text will be ignored! ");
			return false;
		}
		
						//	standard line
		if((count($sbraceo) == 0) && (count($dbraceo) == 0) &&
		   (count($sbracec) == 0) && (count($dbracec) == 0)	)
		{
			(count($amp) == 0) ? $e = count($strings) : $e = $amp[0];
			$newl = array();
			$ret = $this->get_key_value($strings, $start, $eq[0], $e, $newl, $error);
			if($ret == true)
			{
				$k = key($newl);
				$line[$k] = $newl[$k];
			}
			return true;
		}

		if(!((count($sbraceo) == count($sbracec)) &&
		   (count($dbraceo) == count($dbracec))))
		{
			$error[] = array($start, self::FATAL_ERROR, " {} do not match, rest of your text will be ignored! ");
			return false;
		}

				//	first keyword must be in inoplugs_multiselect_const::$pass_through_keys
		$query_key = strtolower($strings[$eq[0] - 1]['value']);
		if(!in_array($query_key, inoplugs_multiselect_const::$pass_through_keys))
		{
			$error[] = array($start, self::FATAL_ERROR, " Keyword [$tkey] is missing, rest of your text will be ignored! ");
			return false;
		}
		
				//		scanloop for  select and relation
		
		$i = 0;			//	index of array
		$esep = count($sep);		//	"}}" is last
		$sbrac = 0;
		$seplast = $sep[0];
		$arrlines = array();
		$arrselect = array();

		while($i < $esep)
		{
			$str0 = $sep[$i];
			
			if($str0["sep"] == "}}")
			{			
				break;
			}
			if($str0["sep"] != "=")
			{
				$error[] = array($str0["pos"], self::FATAL_ERROR, " expecting '=', rest of your text will be ignored! ");
				return false;				
			}
			
			if(($i + 1) < $esep)
			{
				$str1 = $sep[$i + 1];
			}
			else
			{
				$str1 = array($str0["pos"], " ");
			}
		
						//	first after 'categoryselection =' must be '{{'
			if($i == 0)
			{
				if($str1["sep"] != "{{")
				{
					$error[] = array($str1["pos"], self::FATAL_ERROR, " expected {{, rest of your text will be ignored! ");
					return false;
				}
				else
				{
					$seplast = $str1;
					$seplast["pos"] += 2;	//	skip {{
					$i += 2;
					continue;
				}
			}
			
			if(!(($str1["sep"] == "&") || ($str1["sep"] == "{") || ($str1["sep"] == "}")))
			{
				$error[] = array($str1["pos"], self::FATAL_ERROR, " expected {, } or &, rest of your text will be ignored! ");
				return false;
			}

			if(($sbrac == 0) && ($str1["sep"] == "}"))
			{
				$error[] = array($str1['pos'], self::FATAL_ERROR, " unexpected }, rest of your text will be ignored! ");
				return false;
			}

			if(($sbrac > 0) && ($str1["sep"] == "{"))
			{
				$error[] = array($str1['pos'], self::FATAL_ERROR, " unexpected {, nesting is not allowed, rest of your text will be ignored! ");
				return false;
			}
			
			if($str1["sep"] == "{")
			{
				if(strtoupper($strings[$str0['pos']-1]['value']) != strtoupper(inoplugs_multiselect_const::qu_select))
				{
					$error[] = array($str0['pos'] - 1, self::WARNING_ERROR, " expecting keyword '".inoplugs_multiselect_const::qu_select."'! ");
				}
				$sbrac++;
			}
			else if(($str1["sep"] == "&") || (($str1["sep"] == "}")))		//	key = value
			{
				$newl = array();
				$this->get_key_value($strings, $seplast['pos'], $str0['pos'], $str1['pos'], $newl, $error);
				$k = key($newl);

				if($sbrac == 0)		//	relation
				{
					if(strtoupper($k) != strtoupper((inoplugs_multiselect_const::qu_relation)))
					{
						$error[] = array($str0['pos'] - 1, self::WARNING_ERROR, " expecting keyword '".inoplugs_multiselect_const::qu_relation."'! ");
					}				
			
					$arrlines[$k] = $newl[$k];
				}
				else
				{
					$arrselect[$k] =  $newl[$k];
					if($str1["sep"] == "}")
					{
						$arrlines[] = $arrselect;
						$sbrac --;
						$arrselect = array();
					}
				}	
			}
			$seplast = $str1;
			$seplast['pos'] ++;			//	skip & or }
			$i += 2;
		}

		$line[$query_key] = $arrlines;
		return true;
	}

	/**
	 *	Splits the text "key" = "value, value, ..." into an array and fills it in $line:
	 *			Key => array (values)
	 *
	 * @param array $strings		parsed stringarray <array> from content
	 * @param int $start			index,where line starts
	 * @param int $poseq			index of "="
	 * @param int $end				index of last element
	 * @param array $line			"key", array of values
	 * @param array $error			errormessage (index, string) of found error
	 * @return bool				true, if $line could be filled
	 *								(currently always returns true)
	 */
	protected function get_key_value(&$strings, $start, $poseq, $end, &$line, &$error)
	{
							//	Errorchecking and reporting
		if(($poseq) != ($start + 1))		//	must be on second position
		{
			$dif = $poseq - $start - 1;
			$error[] = array($start, self::WARNING_ERROR,
						$dif." next word(s) will be ignored. Only one keyword allowed");
		}

		$key = $strings[$poseq - 1]['value'];
		$a = $poseq + 1;
		
		$val = array();
		while ($a < $end)
		{
			$v = $strings[$a]['value'];
			if(!in_array($v, $this->scqs_sepstring))
			{
				$val[] = $v;
			}
			$a++;
		}
		$line[$key] = $val;
		return true;
	}


}
?>
